//  ------------------------------------------------------------------------

TP.shell.TSH.Inst.defineMethod('executeCli',
function(aRequest) {

    /**
     * @method executeCli
     * @summary
     * @param {TP.sig.ShellRequest} aRequest The request which triggered this
     *     command.
     * @returns {TP.sig.Request} The request.
     */

    var cmd,
        realCmd,

        shell,

        spawnArgs,

        argv,
        seenKeys,
        args;

    if (TP.sys.cfg('boot.context') === 'electron') {

        cmd = this.getArgument(aRequest, 'ARG0', null, false);
        if (TP.isEmpty(cmd)) {
            cmd = 'help';
            realCmd = false;
        } else {
            realCmd = true;
        }

        shell = TP.shell.TSH.getDefaultInstance();

        //  ---
        //  Assemble the spawn arguments
        //  ---

        spawnArgs = TP.ac();

        argv = this.getArgument(aRequest, 'ARGV');
        if (TP.notEmpty(argv)) {

            argv.shift();       //  pop off the first one, it's the command.

            if (TP.notEmpty(argv)) {
                argv.forEach(
                        function(item, ind) {
                            spawnArgs.push(item);
                        });
            }
        }

        seenKeys = TP.hc();

        args = this.getArguments(aRequest);

        //  If there is a real command (which there is in all cases except when
        //  the command is 'help'), filter argN variants. We don't want to have
        //  duplicate arguments sent over the wire.
        if (realCmd) {
            args = args.select(
                    function(kvPair) {
                        if (/ARG\d/.test(kvPair.first()) ||
                            /tsh:ARG\d/.test(kvPair.first())) {
                            if (TP.isValid(args.at('ARGV'))) {
                                args.at('ARGV').shift();
                            }

                            return false;
                        }

                        return true;
                    });
        }

        args.perform(
            function(arg) {
                var key,
                    value;

                key = arg.first();
                value = arg.last();

                //  We already processed ARGV above, which includes all ARG*
                //  arguments.
                if (/^ARG/.test(key)) {
                    return;
                }

                //  Have to get a little fancier here. If we see tsh: prefixes we
                //  want to remove those. If we don't see a value that key is a
                //  boolean flag.
                if (/tsh:/.test(key)) {
                    key = key.slice(4);
                }

                if (seenKeys.at(key)) {
                    return;
                }

                seenKeys.atPut(key, true);

                //  NB: The following logic matches that of the 'cli' plugin for
                //  the TDS.

                spawnArgs.push('--' + key);

                //  We don't add the value if it is null, undefined,
                //  true, the empty string or if the key equals the
                //  value (like a good XML shell, TSH will duplicate
                //  the key name as the value for Boolean flags:
                //  --no-color="no-color").
                if (value !== null &&
                    value !== undefined &&
                    value !== 'true' &&
                    value !== '' &&
                    value !== key) {

                    spawnArgs.push(('' + value).stripEnclosingQuotes());
                }
            });

        TP.extern.electron_lib_utils.commandSpawn(cmd, spawnArgs).then(
            function() {
                console.log('GOT HERE');
            });
    } else {
        this.$processRemoteRequest(aRequest);
    }

    return aRequest;
});


