//  ========================================================================
/**
 * @copyright Copyright (C) 1999 Technical Pursuit Inc. (TPI) All Rights
 *     Reserved. Patents Pending, Technical Pursuit Inc. Licensed under the
 *     OSI-approved Reciprocal Public License (RPL) Version 1.5. See the RPL
 *     for your rights and responsibilities. Contact TPI to purchase optional
 *     privacy waivers if you must keep your TIBET-based source code private.
 */
//  ========================================================================

/**
 * @type {TP.sherpa.styletuner}
 */

//  ------------------------------------------------------------------------

TP.sherpa.Element.defineSubtype('styletuner');

//  ------------------------------------------------------------------------
//  Type Methods
//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Type.defineMethod('getSupportedCSSProperties',
function(aSignal) {

    var props;

    props = TP.ac(
        'animation-direction',
        'backface-visibility',
        'background-color',
        'background-image',
        'border-bottom-color',
        'border-bottom-style',
        'border-bottom-width',
        'border-collapse',
        'border-image-source',
        'border-left-color',
        'border-left-style',
        'border-left-width',
        'border-right-color',
        'border-right-style',
        'border-right-width',
        'border-top-color',
        'border-top-style',
        'border-top-width',
        'bottom',
        'box-sizing',
        'caret-color',
        'clear',
        'color',
        'column-rule-color',
        'direction',
        'display',
        'empty-cells',
        'float',
        'font-style',
        'font-weight',
        'height',
        'left',
        'line-height',
        'list-style-image',
        'list-style-position',
        'margin-bottom',
        'margin-left',
        'margin-right',
        'margin-top',
        'object-position',
        'opacity',
        'outline-color',
        'outline-style',
        'overflow-anchor',
        'overflow-x',
        'overflow-y',
        'padding-bottom',
        'padding-left',
        'padding-right',
        'padding-top',
        'position',
        'resize',
        'right',
        'shape-outside',
        'text-align',
        'text-decoration-color',
        'text-decoration-style',
        'text-transform',
        'top',
        'transform',
        'transition-duration',
        'vertical-align',
        'visibility',
        'white-space',
        'width'
    );

    return props;
});

//  ------------------------------------------------------------------------
//  Instance Attributes
//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Inst.defineAttribute('curtain',
    TP.xpc('//*[@id="systemCurtain"]',
        TP.hc('shouldCollapse', true)
    ).set('fallbackWith', function(tpTarget) {
        return TP.xctrls.curtain.getSystemCurtainFor(
            tpTarget.getDocument(),
            'systemCurtain');
    }));

TP.sherpa.styletuner.Inst.defineAttribute('$propertyName');

TP.sherpa.styletuner.Inst.defineAttribute('currentTargetTPElement');
TP.sherpa.styletuner.Inst.defineAttribute('currentModifyingRule');

TP.sherpa.styletuner.Inst.defineAttribute('propname',
    TP.cpc('> .prop_name', TP.hc('shouldCollapse', true)));

TP.sherpa.styletuner.Inst.defineAttribute('proptuner',
    TP.cpc('> .prop_tuner', TP.hc('shouldCollapse', true)));

//  ------------------------------------------------------------------------
//  Instance Methods
//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Inst.defineHandler('ChangeValue',
function(aSignal) {

    /**
     * @method handleChangeValue
     * @summary Handles notifications of when the receiver wants to change the
     *     value of the CSS property of the current halo target it is
     *     representing.
     * @param {TP.sig.ChangeValue} aSignal The TIBET signal which triggered this
     *     method.
     * @returns {TP.sherpa.styletuner} The receiver.
     */

    var modifyingRule,
        val;

    modifyingRule = this.get('currentModifyingRule');

    val = TP.wrap(aSignal.getDOMTarget()).getValue();

    //  After 100ms, go ahead and complete updating the rule. This usually
    //  entails updating the source document that rule is defined in.
    setTimeout(
        function() {
            TP.styleRuleSetProperty(
                    modifyingRule, this.$get('$propertyName'), val, true);
        }.bind(this), 100);

    return this;
});

//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Inst.defineHandler('CloseTuner',
function(aSignal) {

    this.setAttribute('hidden', true);

    this.detach();

    return this;
});

//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Inst.defineHandler('StartAdjusting',
function(aSignal) {

    /**
     * @method handleStartAdjusting
     * @summary Handles notification of when the receiver wants to start
     *     adjusting it's value. For this type, this means that the 'drag to
     *     adjust' session should start.
     * @param {TP.sig.StartAdjusting} aSignal The TIBET signal which triggered
     *     this method.
     * @returns {TP.sherpa.styletuner} The receiver.
     */

    var nativeEvt,

        curtainTPElem,

        adjuster,
        ourAdjusterEditorTPElem;

    //  Grab the current X from the native Event's screenX and set that to be
    //  the 'last X'.
    nativeEvt = aSignal.at('trigger').getPayload();
    //this.$set('$lastX', TP.eventGetScreenXY(nativeEvt).first());

    //  Grab the curtain as specified by our instance attribute, configure it to
    //  be a 'tracking drag layer' and have it's own cursor.
    curtainTPElem = this.get('curtain');
    curtainTPElem.setAttribute('tibet:isdraglayer', true);
    curtainTPElem.setStyleProperty('cursor', 'ew-resize');

    //  Observe the DOMDragMove and DOMDragUp signals coming from the curtain to
    //  make adjustments as our drag session progresses and then to end it.
    this.observe(
        curtainTPElem, TP.ac('TP.sig.DOMDragMove', 'TP.sig.DOMDragUp'));

    //  Show the curtain.
    curtainTPElem.setAttribute('hidden', false);

    return this;
});

//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Inst.defineHandler('DOMDragMove',
function(aSignal) {

    /**
     * @method handleDOMDragMove
     * @summary Handles notification of when the 'drag to adjust' session is
     *     in process and the value should be adjusted.
     * @param {TP.sig.DOMDragMove} aSignal The TIBET signal which triggered
     *     this method.
     * @returns {TP.sherpa.styletuner} The receiver.
     */

    var nativeEvt,

        lastX,
        currentX,

        direction;

    //  Grab where the X coordinate was last.
    lastX = this.$get('$lastX');

    //  Grab the current X from the native Event's screenX.
    nativeEvt = aSignal.getPayload();
    currentX = TP.eventGetScreenXY(nativeEvt).first();

    //  Compute the direction based on the lastX & currentX values.
    if (currentX < lastX) {
        direction = TP.LEFT;
    } else if (currentX > lastX) {
        direction = TP.RIGHT;
    } else {
        direction = TP.NONE;
    }

    //console.log('X is: ' + currentX);

    //  Adjust the value.
    //this.adjustValue(lastX, currentX, direction, aSignal);

    //  Update the rule. Note that this method does not propagate the rule
    //  changes to the source document. That is done when the rule update is
    //  completed in the drag up handler.
    //this.updateRule();

    //  Reset the last X to whatever we've computed the current X to be.
    //this.$set('$lastX', currentX);

    aSignal.stopPropagation();

    return this;
});

//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Inst.defineHandler('DOMDragUp',
function(aSignal) {

    /**
     * @method handleDOMDragUp
     * @summary Handles notification of when the 'drag to adjust' session is
     *     ending.
     * @param {TP.sig.DOMDragUp} aSignal The TIBET signal which triggered
     *     this method.
     * @returns {TP.sherpa.styletuner} The receiver.
     */

    var adjuster,

        curtainTPElem,

        ourAdjusterEditorTPElem;

    //  Grab the curtain as specified by our instance attribute.
    curtainTPElem = this.get('curtain');

    //  Hide the curtain.
    curtainTPElem.setAttribute('hidden', true);

    //  Remove the attribute that configures the curtain as a 'tracking drag
    //  layer' and clear the 'cursor' property that we set when the drag session
    //  started.
    curtainTPElem.removeAttribute('tibet:isdraglayer');
    curtainTPElem.clearStyleProperty('cursor');

    //  Ignore the DOMDragMove and DOMDragUp signals coming from the curtain
    //  now that our drag session is done.
    this.ignore(
        curtainTPElem, TP.ac('TP.sig.DOMDragMove', 'TP.sig.DOMDragUp'));

    //  Null out the 'last X' so that it's ready for the next dragging session.
    //this.$set('$lastX', null);

    return this;
});

//  ------------------------------------------------------------------------

TP.sherpa.styletuner.Inst.defineMethod('setCssPropertyName',
function(propertyName) {

    /**
     * @method setCSSPropertyName
     * @summary
     * @param {String} propertyName
     * @returns {TP.sherpa.styletuner} The receiver.
     */

    var targetElem,

        propVal,
        propType,

        templateURL,
        resp,
        propTPTemplateElem,

        propTPElem;

    targetElem = this.get('currentTargetTPElement').getNativeNode();

    propVal = targetElem.computedStyleMap().get(propertyName);

    switch (propertyName) {
        case 'background-color':
        case 'border-bottom-color':
        case 'border-left-color':
        case 'border-right-color':
        case 'border-top-color':
        case 'caret-color':
        case 'color':
        case 'column-rule-color':
        case 'outline-color':
        case 'text-decoration-color':
            propType = 'color';
            break;

        case 'border-bottom-width':
        case 'border-left-width':
        case 'border-right-width':
        case 'border-top-width':
        case 'bottom':
        case 'height':
        case 'left':
        case 'line-height':
        case 'margin-bottom':
        case 'margin-left':
        case 'margin-right':
        case 'margin-top':
        case 'padding-bottom':
        case 'padding-left':
        case 'padding-right':
        case 'padding-top':
        case 'right':
        case 'top':
        case 'width':
            propType = 'length';
            break;

        default:
            propType = 'all';
            break;
    }

    templateURL = TP.uc(
                    TP.objectGetSourceCollectionPath(this) + '/' +
                    'property_templates.xml' +
                    '#xpath1(//$def:template[@for="' + propType + '"])');

    resp = templateURL.getResource(
            TP.request('async', false,
                        'resultType', TP.WRAP,
                        'shouldSignal', false));

    propTPTemplateElem = resp.get('result');

    if (TP.isValid(propTPTemplateElem)) {
        this.$set('$propertyName', propertyName);
        this.get('propname').setContent(propertyName + ': ');

        propTPElem = propTPTemplateElem.getFirstChildElement().clone();

        if (propType === 'length') {
            propTPElem.setContent('0px');
        }

        this.get('proptuner').setContent(propTPElem);
    }

    return this;
});

//  ------------------------------------------------------------------------
//  end
//  ========================================================================
